//
//  OTPSOCK
//
//	Copyright (C) 1996-1999 NEC Corporation
//	All rights reserved.
//  Authos(s): Mizukoshi, Yasuhiro <mizukosi@ccs.mt.nec.co.jp>
//
//

//	    $Archive: /sotools/otpsock/common.c $
//			$Revision: 4 $
//			$Date: 99/01/06 11:54a $
static char VssID[] = "$Header: /sotools/otpsock/common.c 4     99/01/06 11:54a Mizukosi $";

#include <windows.h>
#include <winsock.h>

#include <stdio.h>

#ifdef OTPWSK32
#include "common.h"
#include "wsock32/otpwsk32.h"
#include "wsock32/resource.h"
#endif
#ifdef TTXOTP
#include "ttxotp/teraterm.h"
#include "ttxotp/tttypes.h"
#include "ttxotp/ttplugin.h"
#include "common.h"
#include "ttxotp/ttxotp.h"
#include "ttxotp/resource.h"
#endif

///////////////////////////////////////////
//
//	OTP Response Keyword
//
struct KeywordPattern keys[] = {
	{ "s/key",     MD4,  TRUE  },
	{ "otp-md4",   MD4,  FALSE },
	{ "otp-md5",   MD5,  FALSE },
	{ "otp-sha1",  SHA1, FALSE },
	{ "S/Key MD4", MD4,  TRUE  },
	{ "S/Key MD5", MD5,  TRUE  },
	{ "Skey Challenge: s/key", MD4, TRUE },
	{ NULL, 0, FALSE },
};
struct KeywordPattern ftpkeys[] = {
	{ "331 s/key",    MD4,  TRUE  },
	{ "331 otp-md4",  MD4,  FALSE },
	{ "331 otp-md5",  MD5,  FALSE },
	{ "331 otp-sha1", SHA1, FALSE },
	{ "331 OTP response otp-md4",  MD4,  FALSE },
	{ "331 OTP response otp-md5",  MD5,  FALSE },
	{ "331 OTP response otp-sha1", SHA1, FALSE },
	{ "331 Response to otp-md4",   MD4,  FALSE },
	{ "331 Response to otp-md5",   MD5,  FALSE },
	{ "331 Response to otp-sha1",  SHA1, FALSE },
	{ NULL, 0, FALSE },
};
struct KeywordPattern *optkeys = NULL;


////////////////////////////////////////////////
//	connect
//
int PASCAL FAR otp_connect (SOCKET s, const struct sockaddr FAR *name, int namelen)
{
	int		ret;

x_OutputDebugString("connect ");
#if defined(TTXOTP) && defined(_DEBUG)
printf("connect ");
#endif

	// Detect sending password mode
	SendPassMode(name, namelen);

	ret = hook.connect(s, name, namelen);

	return ret;
}


/////////////////////////////////////////////////
//	recv
//
int PASCAL FAR otp_recv (SOCKET s, char FAR * buf, int len, int flags)
{
	int		ret;

#ifdef OTPWSK32
	struct LocalData *ld;
	ld = (struct LocalData *)TlsGetValue(TlsIndex);
#endif

x_OutputDebugString("recv ");
#if defined(TTXOTP) && defined(_DEBUG)
printf("recv ");
#endif

#ifdef OTPWSK32
	if (!ld->bInCall){
		ld->bInCall = TRUE;
#endif

		ret = hook.recv(s, buf, len, flags);

		// scan data
		if (ret != SOCKET_ERROR)
			otpmain(s, buf, ret, flags);

#ifdef OTPWSK32
		ld->bInCall = FALSE;

	} else {
		// recursive call
		ret = hook.recv(s, buf, len, flags);
	}
#endif

	return ret;
}


/////////////////////////////////////////////
//	send
//
int PASCAL FAR otp_send (SOCKET s, const char FAR * buf, int len, int flags)
{
	int		ret;

#ifdef OTPWSK32
	struct LocalData *ld;
	ld = (struct LocalData *)TlsGetValue(TlsIndex);
#endif

x_OutputDebugString("send ");
#if defined(TTXOTP) && defined(_DEBUG)
printf("send ");
#endif

	// If ftp-mode is set, passphrase is sent before sending data.
	if (gOTPData.bSendPass
#ifdef OTPWSK32
		&& !ld->bInCall){
#else
		){
#endif
		char *p;
		int  sent, total;
		char sbuf[ONETIME_KEY_MAX_LEN + 8];

		wsprintf(sbuf, "PASS %s\r\n", gOTPData.pass);

		p     = sbuf;
		total = strlen(sbuf);
		do {
#ifdef OTPWSK32
			ld->bInCall = TRUE;
#endif
			sent = hook.send(s, p, total, flags);
#ifdef OTPWSK32
			ld->bInCall = FALSE;
#endif

			total -= sent;
			p     += sent;
		} while (total) ;

		gOTPData.bSendPass = FALSE;

		return len;
	}

	ret = hook.send(s, buf, len, flags);

	return ret;
}


//////////////////////////////////////////////////////////////
//
//	Passphrase Input Dialog Procedure
//
BOOL CALLBACK PassphraseDlgProc(HWND hWnd, UINT uMessage,
				WPARAM wParam, LPARAM lParam)
{
    switch(uMessage) {
    case WM_INITDIALOG:
	{
		char Title[SEED_MAX_LEN + 10];
		POINT pt, spt;
		RECT  r;

		wsprintf(Title, "%d %s", gOTPData.seq, gOTPData.seed);
		SetWindowText(hWnd, Title);		

		SetWindowText(GetDlgItem(hWnd, IDC_FUNC), gOTPData.FuncName);
		CheckDlgButton(hWnd, IDC_TELNET, (gOTPData.mode - FTP_MODE));

		GetCursorPos(&pt);
		GetWindowRect(hWnd, &r);
		spt.x = GetSystemMetrics(SM_CXSCREEN) - pt.x - (r.right - r.left);
		spt.y = GetSystemMetrics(SM_CYSCREEN) - pt.y - (r.bottom - r.top);
		if (spt.x < 0) pt.x += spt.x;
		if (spt.y < 0) pt.y += spt.y;

		MoveWindow(hWnd, pt.x, pt.y, r.right - r.left, r.bottom - r.top, TRUE);
		ShowWindow(hWnd, SW_SHOWNORMAL);
		SetForegroundWindow(hWnd);
	}
		break;

	case WM_COMMAND:
		switch(wParam){
		case IDOK:
		{
			char passphrase[PASS_PHRASE_MAX_LEN];
			char key[8];
			int  i;

			GetDlgItemText(hWnd, IDC_PASS, passphrase, PASS_PHRASE_MAX_LEN);
			gOTPData.mode = IsDlgButtonChecked(hWnd, IDC_TELNET) + FTP_MODE;

			if (gOTPData.KeyCrunchFunc(key, gOTPData.seed, passphrase) == -1){
				memset(passphrase, 0, PASS_PHRASE_MAX_LEN);
				EndDialog(hWnd, FALSE);
				break;
			}
			for (i = 0; i < gOTPData.seq; i++) gOTPData.IterateFunc(key);
			OTP_btoe(gOTPData.pass, key);

			memset(passphrase, 0, PASS_PHRASE_MAX_LEN);
			EndDialog(hWnd, TRUE);
		}
			break;

		case IDCANCEL:
			EndDialog(hWnd, FALSE);
			break;

		default:
			return FALSE;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}																	   	


////////////////////////////////////////////////
//
//	Check Port Number
//
int SendPassMode(const struct sockaddr FAR *name, int namelen)
{
	short	port;
x_OutputDebugString("SendPassMode:");

	if (name != NULL && namelen == sizeof(struct sockaddr_in)){

		port = ntohs(((struct sockaddr_in *)name)->sin_port);

		if (port == IPPORT_FTP ||
			port == 110 /* POP3 */){
			gOTPData.mode = FTP_MODE;
x_OutputDebugString("FTP_MODE");
			return TRUE;
		}
	}

	gOTPData.mode = TELNET_MODE;
x_OutputDebugString("TELNET_MODE");
	return TRUE;
}


///////////////////////////////////////////
//
//	Set Function Name & Pointers
//
int SetFuncName()
{
	switch(gOTPData.hash_func){
	case MD4:
		gOTPData.KeyCrunchFunc = (void *)_otp_MD4_keycrunch;
		gOTPData.IterateFunc   = (void *)_otp_MD4_iterate;
		strcpy(gOTPData.FuncName, FUNCNAME_MD4);
		break;
	case MD5:
		gOTPData.KeyCrunchFunc = (void *)_otp_MD5_keycrunch;
		gOTPData.IterateFunc   = (void *)_otp_MD5_iterate;
		strcpy(gOTPData.FuncName, FUNCNAME_MD5);
		break;
	case SHA1:
		gOTPData.KeyCrunchFunc = (void *)_otp_SHA1_keycrunch;
		gOTPData.IterateFunc   = (void *)_otp_SHA1_iterate;
		strcpy(gOTPData.FuncName, FUNCNAME_SHA1);
		break;

	default:
		return FALSE;
	}

	return TRUE;
}


////////////////////////////////////////
//
//	Keyword Search
//
int KeywordSearch(struct KeywordPattern *kp)
{
	int		i;

	if (kp == NULL)
		return FALSE;

	for (i = 0; kp[i].key != NULL; i++){

		if ( !strncmp(gOTPData.recvbuf, kp[i].key, strlen(kp[i].key)) ){
			gOTPData.hash_func = kp[i].func;

			// for S/Key MD5
			if ( kp[i].IsSKey && gbSKeyMD5 )
				gOTPData.hash_func = MD5;

			SetFuncName();

			if ( sscanf(gOTPData.recvbuf + strlen(kp[i].key), SEQSEED_POS, &gOTPData.seq, gOTPData.seed) != 2 ){
				return FALSE;
			}

			return TRUE;

		}
	}

	return FALSE;
}


////////////////////////////////////////
//
//	Check Response
//
int chkResp()
{
	struct KeywordPattern *kp;

	if ( !strncmp(gOTPData.recvbuf, FTP_RESPONSE, strlen(FTP_RESPONSE)) ){
		kp = ftpkeys;
	} else {
		kp = keys;
	}

	if (KeywordSearch(kp) || KeywordSearch(optkeys))
		return TRUE;
	else
		return FALSE;
}


/////////////////////////////////////////////////////////
//
//	OTP Main
//		SOCKET		s,		socket
//		char FAR	*buf,	receive data buffer
//		int			len		receive data length
//		int			flags	flag of send()
//
int otpmain(SOCKET s, char FAR * buf, int len, int flags)
{
	int		i;
	int		res;

	for (i = 0; i < len; i++){

		// Over flow
		if (gOTPData.recvbuf_pos >= MAX_OTPBUF_SIZE){
			gOTPData.recvbuf[0]  = 0;
			gOTPData.recvbuf_pos = 0;
		}

		gOTPData.recvbuf[gOTPData.recvbuf_pos] = buf[i];

		if (gOTPData.recvbuf[gOTPData.recvbuf_pos] == '\n'){
			// Set NULL for end-of-string
			gOTPData.recvbuf[gOTPData.recvbuf_pos] = 0;
			gOTPData.recvbuf_pos = 0;

		} else if (iReqEndLen &&
				   iReqEndLen <= (gOTPData.recvbuf_pos + 1) &&
				   !strncmp((gOTPData.recvbuf+gOTPData.recvbuf_pos+1-iReqEndLen), szReqEnd, iReqEndLen)){
			gOTPData.recvbuf[gOTPData.recvbuf_pos+1] = 0;
			gOTPData.recvbuf_pos++;

		} else {
			gOTPData.recvbuf_pos++;
			continue;
		}

		// Chech whether recv data includes challenges.
		if ( chkResp() ){
			HWND	hWndDotkey;

			// Find OTP calculater
			hWndDotkey = FindWindow(NULL, gOTPCalcWndName);

			if (hWndDotkey){

				res = FALSE;

				wsprintf(gtbuf, OTPCALC_SERVER_TMPL,
						gOTPData.FuncName,
						gOTPData.seq,
						gOTPData.seed);

				gCDS.dwData = 0;
				gCDS.cbData = MAX_OTPBUF_SIZE + 1;
				gCDS.lpData = gtbuf;

				// Copy challenges to OTP calculater
				SendMessage(hWndDotkey, WM_COPYDATA,
							(WPARAM)NULL, (LPARAM)&gCDS);

				Sleep(500);

				// Get result from OTP calculater
				if (OpenClipboard(NULL) == TRUE){
					HANDLE hClipMem = GetClipboardData(CF_TEXT);

					if (hClipMem != NULL){
						char * pClipMem = (char *)GlobalLock(hClipMem);

						if (pClipMem[0]){
							strcpy(gOTPData.pass, pClipMem);

							res = TRUE;
						}

						GlobalUnlock(hClipMem);
					}
					CloseClipboard();
				}

			} else {
				CheckSeqNum();

				res = DialogBox(ghModule, MAKEINTRESOURCE(IDD_PASSPHRASE),
								NULL, PassphraseDlgProc);

			}

			if (res){
				char *p;
				int  sent, total;
				char sbuf[ONETIME_KEY_MAX_LEN + 8];

				if (gOTPData.mode == FTP_MODE){
					gOTPData.bSendPass = TRUE;

				} else {
					wsprintf(sbuf, "%s\r\n", gOTPData.pass);

					p     = sbuf;
					total = strlen(sbuf);
					do {
						sent = hook.send(s, p, total, flags);

						total -= sent;
						p     += sent;
					} while (total) ;
				}
			}
		}
	}

	return TRUE;
}


/////////////////////////////////////////////////////////
//
//	Check Sequence Number
//
int CheckSeqNum()
{
	char szbuf[128];

	switch (gOTPData.seq){
	case 20: case 10: case  5: case  4:
	case  3: case  2: case  1: case  0:
		wsprintf(szbuf, "***** Warning!!  %d times to go. *****", gOTPData.seq);
		MessageBox(NULL, szbuf, "otpsock", MB_OK | MB_SYSTEMMODAL);
		break;
	default:
		break;
	}

	return TRUE;
}


////////////////////////////////////////////////////////////////
//
//	Initialize Preferences
//
int	InitPreferences()
{
	HKEY  hKey;
	DWORD dwType;
	DWORD dwLength;
	DWORD dwData;
	int   i, ret, cbValue;
	char  szValue[MAX_KEYWORD_LEN];

	/////////////////////////////////
	// Set Default Value
	strcpy(gOTPCalcWndName, DEFAULT_OTPCALC);
	gbSKeyMD5   = FALSE;
	szReqEnd[0] = 0;
	iReqEndLen  = strlen(szReqEnd);


	/////////////////////////////////
	// Load Registory
	if (RegOpenKeyEx(HKEY_CURRENT_USER, OTPSOCK_SUBKEY, 0, KEY_QUERY_VALUE, &hKey) != ERROR_SUCCESS){
		return FALSE;
	}

	// OTPCalcWndName
	dwType   = REG_SZ;
	dwLength = sizeof(gOTPCalcWndName);
	RegQueryValueEx(hKey, OTPCALC_VALNAME, NULL, &dwType, gOTPCalcWndName, &dwLength);

	// bSKeyMD5
	dwType   = REG_DWORD;
	dwLength = sizeof(dwData);
	if (RegQueryValueEx(hKey, SKEY_MD5_VALNAME, NULL, &dwType, (LPBYTE)&dwData, &dwLength) == ERROR_SUCCESS)
		gbSKeyMD5 = (BOOL)dwData;

	// ChallengeEndChar
	dwType	 = REG_SZ;
	dwLength = sizeof(szReqEnd);
	RegQueryValueEx(hKey, REQ_END_VALNAME, NULL, &dwType, szReqEnd, &dwLength);
	iReqEndLen  = strlen(szReqEnd);

	RegCloseKey(hKey);


	///////////////////////////////////////////
	// Load optional challenge keywords
	if (RegOpenKeyEx(HKEY_CURRENT_USER, OTPSOCK_KEYWORDS_SUBKEY, 0, KEY_QUERY_VALUE, &hKey) != ERROR_SUCCESS){
		return FALSE;
	}

	// Get the number of optional challenge keywords
	i = 0;
	while (1){
		cbValue  = sizeof(szValue);
		ret = RegEnumValue(hKey, i, szValue, &cbValue, NULL, NULL, NULL, NULL);
		if (ret != ERROR_SUCCESS)
			break;
		i++;
	}
	if (i == 0){
		goto error;
	} else {
		optkeys = (struct KeywordPattern *)malloc(sizeof(struct KeywordPattern) * (i + 1));
		if (optkeys == NULL)
			goto error;
	}

	// Set value to optkeys variable
	i = 0;
	while (1){
		cbValue  = sizeof(szValue);
		dwType   = REG_DWORD;
		dwLength = sizeof(dwData);

		ret = RegEnumValue(hKey, i, szValue, &cbValue, NULL, &dwType, (LPBYTE)&dwData, &dwLength);

		if (ret != ERROR_SUCCESS)
			break;

		optkeys[i].key = (char *)malloc(cbValue+1);
		if (optkeys[i].key == NULL)
			break;
		strcpy(optkeys[i].key, szValue);

		if (dwData <= MD2) dwData = MD5;
		if (dwData > SHA1) dwData = MD5;
		optkeys[i].func = dwData;

		optkeys[i].IsSKey = FALSE;

		i++;
	}

	// Set the termination to optkeys variable
	optkeys[i].key = NULL;

error:
	RegCloseKey(hKey);
	return TRUE;
}
